//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
#include "Precomp.h"

#ifndef NO_FAVORITES

#include "resource.h"
#include "mainwnd.h"
#include "FavFolders.h"
#include "debug.h"
#include "regkeys.h"
#include "RegFavorites.h"

CFavFoldersDlg::CFavFoldersDlg(CWindow* pMainWnd,int nTitle,int nMessage,BOOL bNewFolder)
{
	m_wndMainWnd   = pMainWnd;			// Main window object pointer
	m_nOpenFolder  = 0;					// Index to the open folder icon
	m_nCloseFolder = 0;					// Index to the close folder icon
	m_hIml         = NULL;
	m_fSrcSelected = FALSE;
    m_nTitle	   = nTitle;
	m_nMessage	   = nMessage;  
	m_bNewFolderSupport = bNewFolder;
	m_pIFavStorage = (CRegFavorites*) new CRegFavorites;	

	INIT_STR (m_szSrcPath);
}

CFavFoldersDlg::~CFavFoldersDlg()
{
	m_wndMainWnd = NULL;
	if (m_hIml)
	{
		ImageList_Destroy(m_hIml);
		m_hIml = NULL;
	}

	if (m_pIFavStorage)
		delete m_pIFavStorage;

	if (m_wndTreeCtrl.m_hWnd)
		TreeView_DeleteAllItems(m_wndTreeCtrl.m_hWnd);
	m_wndTreeCtrl.Detach();
}

LRESULT CFavFoldersDlg::OnCreate (UINT uMsg, WPARAM wParam, LPARAM lParam,BOOL& bHandled)
{
	CREATESTRUCT *lpcs = NULL;
	lpcs = (LPCREATESTRUCT) lParam;
	return 0;
}

LRESULT CFavFoldersDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	TCHAR tszTemp[80];

	if (!m_pIFavStorage)
		return 0;

	CenterWindow ();
	m_wndTreeCtrl.Attach(GetDlgItem(IDC_FAVORITES));

	if (!m_FavHelper.InitHelper (g_hInstance, &m_wndTreeCtrl,m_pIFavStorage,MODE_FAVFOLDERS))
		return 0;

	if (!m_bNewFolderSupport)
		::ShowWindow (GetDlgItem(IDC_CREATE_FOLDER),SW_HIDE);

	if (m_FavHelper.InitTreeViewImageLists())
		m_FavHelper.PopulateTreeControl(m_pIFavStorage->GetRootFolder());
	else
		return 0;

	m_wndTreeCtrl.SetFocus();

	// Expand the tree
	TreeView_Expand (m_wndTreeCtrl.m_hWnd,TVI_ROOT,TVE_EXPAND);

	INIT_STR (tszTemp);
	LoadString (g_hInstance,m_nTitle,tszTemp,sizeof (tszTemp)/sizeof (tszTemp[0]));
	SetWindowText (tszTemp);

	INIT_STR (tszTemp);
	LoadString (g_hInstance,m_nMessage,tszTemp,sizeof (tszTemp)/sizeof (tszTemp[0]));
	SetDlgItemText (IDC_STATIC_MSG,tszTemp);
	
	return 1;
}

LRESULT CFavFoldersDlg::OnCreateFolder (WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	m_FavHelper.CreateFolder (wNotifyCode, wID, hWndCtl, bHandled);
	return 0;
}

LRESULT CFavFoldersDlg::OnBeginEditLabel(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	return m_FavHelper.BeginEditLabel (idCtrl, pnmh, bHandled);
}

LRESULT CFavFoldersDlg::OnEndEditLabel(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	BOOL bFlag = FALSE;
	return m_FavHelper.EndEditLabel (idCtrl, pnmh, bHandled, bFlag, NULL);
}

TCHAR* CFavFoldersDlg::GetDestinationFolder()
{
	DEBUGMSG (ZONE_TRACE, (L"CFavFoldersDlg::GetDestinationFolder\n"));

	if (_tcslen (m_szSrcPath) > 0)
		return m_szSrcPath;

	return NULL;
}

LRESULT CFavFoldersDlg::OnOk (WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled)
{
	HTREEITEM hItem = NULL;
	TVITEM tvi;

	// Get the item selected in tree
	if (NULL == (hItem = TreeView_GetSelection (m_wndTreeCtrl.m_hWnd)))
		wcscpy (m_szSrcPath,m_pIFavStorage->GetRootFolder());
	else
	{
		m_FavHelper.IsLeafItem (hItem, tvi);
		wcscpy (m_szSrcPath,reinterpret_cast<LPTSTR>(tvi.lParam));
	}

	EndDialog (IDOK);
	return 1;
}

LRESULT CFavFoldersDlg::OnCancel (WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled)
{
	EndDialog (0);
	return 1;
}

#endif // ndef NO_FAVORITES

