//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
/*++
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name:  mainwnd.cpp

Abstract:  Implements the main window, the container for the webbrowser

Functions:

Notes: Most of the code resides here. The container and its interaction with the webbrowser control,
        commandbar, statusbar etc.

//Config switches
#define NOSTATUSBAR		//hide statusbar AppendMenu 
#define NOCOMMANDBAR	//hide commandbar
#define NO_FAVORITES	//hide favorites
#define NO_ADDRESSBAR	//hide the address field

#define NOOPTIONS		//hide options from menu
#define NOFIND
#define NOCOMMANDBAR
#define NOPRINT
#define NOOFFLINE
#define NO_CLOSE		//no close item in menu
#define NO_SEARCH		//hide the search button
#define OEMCONTROLS
--*/
#define NOCOMMANDBAR	//hide commandbar
#define NO_MENU			//hide the menu
#define NOSTATUSBAR		//hide statusbar
#define NO_FAVORITES	//hide favorites
#define NO_ADDRESSBAR	//hide the address field
#define NO_SEARCH		//hide the search button

#define USEFULLSCREEN

#include "Precomp.h"
#include "resource.h"
#include "MainWnd.h"
#include "animthrd.h"
#ifndef NO_FAVORITES
	#include "addfavoritesdlg.h"
	#include "favlistfulldlg.h"
	#include "organizefavoritesdlg.h"
	#include "regkeys.h"
	#include "debug.h"
	#include "RegFavorites.h"

#endif

//HJG #include "passwordDlg.h" //HJG

	DBGPARAM dpCurSettings; //HJG moved outside endif

#define INITGUID
#include "initguid.h"

//the password entered in password dialog
#include "pwdialog.h"

static TCHAR pw[]=L"................";

// Forward declarations of functions included in this code module:
//LRESULT CALLBACK	Password		(HWND, UINT, WPARAM, LPARAM);


DEFINE_GUID(CLSID_WebBrowser,       0x8856F961L, 0x340A, 0x11D0, 0xA9, 0x6B, 0x00, 0xC0, 0x4F, 0xD7, 0x05, 0xA2);
DEFINE_GUID(IID_IWebBrowser,        0xEAB22AC1L, 0x30C1, 0x11CF, 0xA7, 0xEB, 0x00, 0x00, 0xC0, 0x5B, 0xAE, 0x0B);
DEFINE_GUID(IID_IWebBrowser2,       0xD30C1661L, 0xCDAF, 0x11D0, 0x8A, 0x3E, 0x00, 0xC0, 0x4F, 0xC9, 0xE2, 0x6E);
DEFINE_GUID(DIID_DWebBrowserEvents, 0xEAB22AC2L, 0x30C1, 0x11CF, 0xA7, 0xEB, 0x00, 0x00, 0xC0, 0x5B, 0xAE, 0x0B);
DEFINE_GUID(DIID_DWebBrowserEvents2, 0x34A715A0L, 0x6587, 0x11D0, 0x92, 0x4A, 0x00, 0x20, 0xAF, 0xC7, 0xAC, 0x4D);
DEFINE_GUID(IID_IWebBrowserApp,     0x0002DF05L, 0x0000, 0x0000, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46);

const GUID SID_SDocHost = { 0xc6504990, 0xd43e, 0x11cf, { 0x89, 0x3b, 0x00, 0xaa, 0x00, 0xbd, 0xce, 0x1a}};


static TBBUTTON tbBtns[] = {    
	{0, ID_GO_BACK, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
    {1, ID_GO_FORWARD, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
    {2, ID_VIEW_STOP, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
    {3, ID_VIEW_REFRESH, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
    {4, ID_GO_HOME, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
#ifndef NO_SEARCH //HJG
    {5, ID_GO_SEARCH, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
#endif
    {15, ID_GO_EXIT, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
};

static TCHAR szTypedUrls[] = _T("Software\\Microsoft\\Internet Explorer\\TypedURLs");

#define PROGRESSPANEWIDTH	200			//width of the progressbar
#define MAX_STATUSMSGLEN	127			//the maximum length for the statusbar message
#define ID_COMBOEDIT		1001		//ID for the edit box of a combo box
#define MAXURLS				25			//maximum number of typed url's.

#define MAX(a,b)  (a > b) ? a : b
#define MIN(a,b)  (a > b) ? b : a

LONG        glThreadCount	= 0;
HWND		ghWndAddressEdit= NULL;
HANDLE      ghExitEvent		= NULL;
HINSTANCE   g_hInstance		= NULL;
DWORD      g_dwMainWindowStackSize = 0x20000;

void GetProxyOption();
#ifdef OEMCONTROLS
BOOL RegisterOemControls();
#endif

static HRESULT FindString();
HRESULT HandleNewWindow2(LPTSTR lpszUrl, DISPPARAMS FAR* pdparams);
BOOL RegisterMainWnd();


DWORD WINAPI NewWindow(LPVOID pParam)
{

    CMainWnd *pWnd = (CMainWnd *)pParam;
    MSG msg;
    BOOL fRet;

    HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

    fRet = pWnd->Create();

    SetEvent(pWnd->hEvent);

    if (!fRet)
    {
        pWnd->_pBrowser = NULL;
        return 0;
    }

    while (GetMessage( &msg, NULL, 0, 0 ) )
    {
        if(msg.message == WM_QUIT)
            break;

        if (!pWnd->PreTranslateMessage(&msg) && !(msg.message == WM_CHAR && msg.wParam == VK_TAB))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    pWnd->Release();

    CoUninitialize();

    InterlockedDecrement(&glThreadCount);
    SetEvent(ghExitEvent);

    return msg.wParam;
}

int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPTSTR lpCmdLine, int nCmdShow)
{
    INITCOMMONCONTROLSEX iccsex;
    HKEY hKey;
    DWORD dwSize = sizeof(DWORD);

    // HKCU is where IE\Main settings are
    if(ERROR_SUCCESS == RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("SOFTWARE\\Microsoft\\Internet Explorer\\Main"), 0, 0, &hKey))
    {
            RegQueryValueEx(hKey, TEXT("StackRes"), NULL, NULL, (LPBYTE)&g_dwMainWindowStackSize, &dwSize);
            RegCloseKey(hKey);
    }

    // provide a default stack size if the one given is too small or too large. 
    if(g_dwMainWindowStackSize < 0x10000 || g_dwMainWindowStackSize > 0x80000)
    {
            // default to 128k
            g_dwMainWindowStackSize = 0x20000;
    }
    RETAILMSG(1,(L"IESAMPLE Using Stack size: %x", g_dwMainWindowStackSize));
#ifndef NO_FAVORITES
	// hook in ATL
	_Module.Init(NULL, reinterpret_cast<HINSTANCE>(hInst));
#endif

    HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
    if (FAILED(hr))
    {
        return FALSE;
    }

#ifdef OEMCONTROLS
    if (!RegisterOemControls())
            return FALSE;
#endif	//OEMCONTROLS

   if (!RegisterMainWnd())
            return FALSE;
    
    ghExitEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (!ghExitEvent)
    {
        return FALSE;
    }

    iccsex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    iccsex.dwICC = ICC_COOL_CLASSES
#ifndef NO_FAVORITES
    |ICC_TREEVIEW_CLASSES
#endif
    ;

    InitCommonControlsEx(&iccsex);

    g_hInstance = hInst;

    if(HandleNewWindow2(lpCmdLine, NULL))
    {
        goto Cleanup;
    }


    while (glThreadCount > 0)
    {
        WaitForSingleObject(ghExitEvent, INFINITE);
    }
    
Cleanup:    
    CoUninitialize();
#ifndef NO_FAVORITES    
    _Module.Term();
#endif    
    RETAILMSG(1, (L"IESAMPLE exited. Cmdline was: %s\r\n",lpCmdLine ? lpCmdLine : L""));

    return TRUE;
}


CMainWnd::CMainWnd()
{
    _ulRefs = 1;
    _hWnd = NULL;
    _pBrowser = NULL;
    _pObject = NULL;
    _pCP = NULL;
    _lpszUrl = NULL;
#ifndef NOCOMMANDBAR
    _lpfnAddressEditProc = NULL;
    _hWndMenuBar = NULL;
    _hWndAddrBar = NULL;
    _hWndToolBar = NULL;
    _hWndBrowser = NULL;
    _himlCmdBand = NULL;
#endif	//NOCOMMANDBAR

#ifndef NOSTATUSBAR
    _hWndStatus = NULL;
    _hWndProgress = NULL;
#endif	//NOSTATUSBAR

    _bTyped = FALSE;

    _szTitle[0] = 0;

#ifndef NO_FAVORITES
        _hMenu = NULL;
	_nNumFavorites = 0;
	_nLastFavoriteID = ID_FAVORITE_FIRST;
	_pIFavStorage = (CRegFavorites*) new CRegFavorites;
#endif
	
}

CMainWnd::~CMainWnd()
{

RETAILMSG(1,(L"IESAMPLE Exiting ~CMainWnd\r\n"));
#ifndef NOCOMMANDBAR
        if(_himlCmdBand)
                ImageList_Destroy(_himlCmdBand);
#endif
        if(_pBrowser)
            _pBrowser->Release();

#ifndef NO_FAVORITES    
	if (_pIFavStorage)
		delete _pIFavStorage;
#endif	
}   

BOOL RegisterMainWnd()
{
    WNDCLASS        wc;
    wc.style         = 0;
    wc.lpfnWndProc   = (WNDPROC)CMainWnd::MainWndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = g_hInstance;
    wc.hIcon         = LoadIcon(g_hInstance, MAKEINTRESOURCE(IDI_IE));
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetSysColorBrush(COLOR_WINDOW);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = TEXT("IESAMPLE");

    if (!(RegisterClass(&wc)))
        return FALSE;

    return TRUE;
}


BOOL CMainWnd::Create()
{
    RECT rcArea, rcWindow;

    DWORD dwStyle = WS_VISIBLE/*|WS_OVERLAPPED|WS_BORDER|WS_CAPTION
                        |WS_SYSMENU|WS_MINIMIZEBOX*/;
    DWORD dwExStyle = 0/*WS_EX_OVERLAPPEDWINDOW*/;

    HMENU hMenu = NULL;


    _hWnd = ::CreateWindowEx(dwExStyle,
                             TEXT("IESAMPLE"),
                             LOADSTRING(IDS_IE),
                             dwStyle,
                             CW_USEDEFAULT,
                             0,
                             CW_USEDEFAULT,
                             0,
                             NULL, hMenu, g_hInstance, 0);
    if (!_hWnd)
        return FALSE;

    SetWindowLong(_hWnd, GWL_USERDATA, (DWORD)this);
#ifndef USEFULLSCREEN
    SystemParametersInfo(SPI_GETWORKAREA, 0, &rcArea, 0);
#else
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rcArea, 0);
	rcArea.bottom=rcArea.bottom+26;
	HWND hwndTaskBar = FindWindow(L"HHTASKBAR", NULL);
	if (hwndTaskBar!=NULL)
	{
		EnableWindow(hwndTaskBar, false);
		SetWindowPos(_hWnd, HWND_TOPMOST, rcArea.left, rcArea.top, rcArea.right, rcArea.bottom, SWP_SHOWWINDOW);
	}
#endif
    GetWindowRect(_hWnd, &rcWindow);

#ifndef NOCOMMANDBAR
    if (!CreateCommandbar())
        return FALSE;

    _pthrdAnimation = new CAnimThread(this);
    if (!_pthrdAnimation)
        return FALSE;

    _pthrdAnimation->StartThread();

#endif	//NOCOMMANDBAR

#ifndef NOSTATUSBAR
    if (!CreateStatusBar())
        return FALSE;
#endif	//NOSTATUSBAR

    if (!(_hWndBrowser = CreateBrowser()))
        return FALSE;

    LONG lStyle = GetWindowLong(_hWndBrowser, GWL_STYLE);
    SetWindowLong(_hWndBrowser, GWL_STYLE, lStyle|WS_BORDER);
 
	SetFocus(_hWnd);

    _hAccelTbl = LoadAccelerators(g_hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR));

    return TRUE;
}

#ifndef NOCOMMANDBAR

///////////////////////////////////////////////////////////////////////////////
// CreateCommandbar()
///////////////////////////////////////////////////////////////////////////////
BOOL CMainWnd::CreateCommandbar()
{
    TBBUTTON tbSep = {0, IDC_STATIC, 0, TBSTYLE_SEP, 0, 0, 0, -1};

    RECT rc;
    GetClientRect(_hWnd, &rc);
    

    HBITMAP hbm;
    
    _himlCmdBand = ImageList_Create(11, 13, ILC_COLOR, 1, 0);
	    hbm = LoadBitmap(g_hInstance, MAKEINTRESOURCE(IDB_COOLBARIMAGES));	
    if (hbm) {
           ImageList_Add(_himlCmdBand, hbm, NULL);
	    DeleteObject(hbm);
    }

    _hWndCmdband = CommandBands_Create(g_hInstance, _hWnd, ID_COMMANDBAND,
                                       RBS_SMARTLABELS|RBS_BANDBORDERS|RBS_AUTOSIZE, _himlCmdBand);
    REBARBANDINFO rbbi[3];


#ifndef NO_ADDRESSBAR
    // Create the Address window
    _hWndAddressCombo = CreateWindowEx(0, TEXT("combobox"), TEXT(""), 
										 WS_VISIBLE | WS_CHILD |WS_TABSTOP| CBS_AUTOHSCROLL,
										 0, 0, 20, 110, _hWnd, 
										 (HMENU)ID_ADDRESSBAR,
										 g_hInstance, NULL);

    
    LoadTypedUrls();

    _hWndAddressEdit = GetDlgItem(_hWndAddressCombo, ID_COMBOEDIT);
	ghWndAddressEdit = _hWndAddressEdit;

    SetWindowLong(_hWndAddressEdit, GWL_USERDATA, (DWORD)this);

    _lpfnAddressEditProc = (WNDPROC )SetWindowLong(_hWndAddressEdit,
                                                   GWL_WNDPROC, (LONG)AddressEditProc);
    SendMessage(_hWndAddressEdit, EM_LIMITTEXT, MAXLEN-1, 0L);
#endif //NO_ADDRESSBAR

//HJG to hide the menubar rearrange the number of commandbars
	int ix=0;
#ifndef NO_MENU
    // Menubar
    rbbi[ix].cbSize = sizeof(REBARBANDINFO);
    rbbi[ix].fMask = RBBIM_ID|RBBIM_IMAGE|RBBIM_SIZE; 
    rbbi[ix].cx = 250;  // want to be able to size this automatically
    rbbi[ix].wID = ID_BAND_CMD;
    rbbi[ix].iImage = 1;
	ix++;
#endif
    // Toolbar
    rbbi[ix].cbSize = sizeof(REBARBANDINFO);
    rbbi[ix].fMask = RBBIM_ID|RBBIM_IMAGE|RBBIM_SIZE;
    rbbi[ix].wID = ID_BAND_TOOL;
    rbbi[ix].iImage = 0;
    rbbi[ix].cx = rc.right - (ANIMATIONWIDTH + 250);
	// want to be able to size this automatically

#ifndef NO_ADDRESSBAR
	ix++;

    // Address
    WCHAR szAddressLabel[40];
    RECT rcComboBox;
    GetWindowRect(_hWndAddressCombo, &rcComboBox);
    

    LoadString(g_hInstance, IDS_ADDRESS, szAddressLabel, 40);		
    rbbi[ix].cbSize = sizeof(REBARBANDINFO);
    rbbi[ix].fMask = RBBIM_CHILD | RBBIM_TEXT | RBBIM_ID | RBBIM_STYLE | 
						 RBBIM_CHILDSIZE | RBBIM_IMAGE;
    rbbi[ix].wID = ID_BAND_ADDR;
    rbbi[ix].fStyle = RBBS_BREAK;
    rbbi[ix].cxMinChild = 0;
    rbbi[ix].cyMinChild = rcComboBox.bottom - rcComboBox.top; 
    rbbi[ix].iImage = 2;
    rbbi[ix].hwndChild = _hWndAddressCombo;
    rbbi[ix].lpText = (LPWSTR)szAddressLabel;
#endif //NO_ADDRESSBAR
    
    if (!CommandBands_AddBands(_hWndCmdband, g_hInstance, ix+1, rbbi))
        return FALSE;

#ifndef NO_MENU
    _hWndMenuBar = CommandBands_GetCommandBar(_hWndCmdband, 0);

    if (!CommandBar_InsertMenubar(_hWndMenuBar, g_hInstance, IDR_MAINMENU, 0))
        return FALSE;
        
#ifndef NO_FAVORITES

	PopulateFavoritesMenu();
	// erase the separator and gray delete menu item if there are no favorites
	if (_nNumFavorites == 0)
	{
		HMENU hMenu = GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), FAVORITES_MENU_POSITION);
		EnableMenuItem(hMenu, ID_FAV_PLACEHOLDER, MF_BYCOMMAND | MF_GRAYED);
	} // if
	else
		DeleteMenu (GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), FAVORITES_MENU_POSITION),
						ID_FAV_PLACEHOLDER, MF_BYCOMMAND);
#endif //NO_FAVORITES
#endif //NO:MENU

	_hWndToolBar = CommandBands_GetCommandBar(_hWndCmdband, ix); //use ix = last band created

    if ((CommandBar_AddBitmap(_hWndToolBar, g_hInstance, IDB_TBBITMAP,
                              sizeof(tbBtns)/sizeof(TBBUTTON), 16, 16)) < 0)
        return FALSE;

    if (!CommandBar_AddButtons(_hWndToolBar, _countof(tbBtns), tbBtns))
        return FALSE;

    if (!CommandBar_InsertButton(_hWndToolBar, 3, &tbSep))
        return FALSE;

    return TRUE;


}

#ifndef NO_FAVORITES
UINT CMainWnd::PopulateFavoritesMenu()
{
	UINT  uID	   = ID_FAVORITE_FIRST;
	HMENU hFavMenu = NULL;

	if (!_pIFavStorage)
		return 0;

	// Continue only if Favorites registry key is present 
	if (!_pIFavStorage->IsFavAvailable())
		return 0;

    if (NULL == (hFavMenu = GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), 
									   FAVORITES_MENU_POSITION)))
		return 0;

	if (!InsertFavoriteIntoMenu(hFavMenu,_pIFavStorage->GetRootFolder(),uID))
		return 0;

	return _nNumFavorites;
} // PopulateFavoritesMenu
#endif  // ndef NO_FAVORITES
#endif	//NOCOMMANDBAR

#ifndef NO_FAVORITES
BOOL CMainWnd::InsertFavoriteIntoMenu(HMENU hMenu, LPTSTR szPath, UINT uID, UINT uPos)
{
	HTREEITEM   hParent     = NULL;
	DWORD       dwFolders   = 0;
	DWORD       dwRegKeyLen = 0;
	DWORD       idx			= 0;
	DWORD       dwValues    = 0;
	static UINT  _uID	    = uID;
	BOOL		bRetVal     = TRUE;
	TCHAR       szRegKey1[MAX_REGKEY_LEN];
	TCHAR       szRegKey [FAV_NAME_LEN];

	if (!_pIFavStorage)
		return FALSE;

	INIT_STR (szRegKey);
	INIT_STR (szRegKey1);

	if (ERROR_SUCCESS != _pIFavStorage->GetFolderInfo (szPath,
													    dwFolders,
														dwValues))
		goto ERR_RET;

	if (!dwFolders)		// If there are no folders
	{
		// If this item has no values also, it is assumed to be an empty folder.
		// In this case, an empty sub folder is added to this menu item.
		if (!dwValues)
			GOERROR(hMenu = SetMenuInfo (hMenu,_uID,uPos));
		
		goto RETURN;
	}
	else
		if (_uID != ID_FAVORITE_FIRST)
			GOERROR(hMenu = SetMenuInfo (hMenu,_uID,uPos));

	for (idx=0; idx < dwFolders; idx++)
	{
		dwRegKeyLen = FAV_NAME_LEN-1;
		if (ERROR_SUCCESS != _pIFavStorage->EnumFolders (szPath,
														  idx,
														  szRegKey,
														  dwRegKeyLen))
			continue;

		GOERROR (AddMenuItem(hMenu,_uID,szRegKey,szPath));
		++_uID;
		wsprintf(szRegKey1,L"%s\\%s",szPath,szRegKey);
		InsertFavoriteIntoMenu(hMenu,szRegKey1,_uID,idx);
	}
	goto RETURN;

ERR_RET:

	bRetVal = FALSE;
	
RETURN:
	if (!_tcscmp (_pIFavStorage->GetRootFolder(), szPath))
		_uID = ID_FAVORITE_FIRST;

	return bRetVal;
}

BOOL CMainWnd::DeleteMenuItem (HMENU hMenu, UINT uPos)
{
	MENUITEMINFO menuInfo;

	for (UINT idx=uPos;;idx++)
	{
		menuInfo.cbSize = sizeof (menuInfo);
		menuInfo.fMask  = MIIM_SUBMENU;
		if (GetMenuItemInfo (hMenu, idx, TRUE, &menuInfo))
		{
			if (menuInfo.hSubMenu)
				DestroyMenu (menuInfo.hSubMenu);
			DeleteMenu (hMenu, idx, MF_BYPOSITION);
			idx -= 1;
		}
		else
			break;
	}
	return TRUE;
}

BOOL CMainWnd::AddMenuItem (HMENU hMenu, UINT &uID, LPCTSTR szText, LPCTSTR szInfo)
{
	MENUITEMINFO menuInfo;
	BOOL		 bSetInfo = FALSE;
	LPTSTR		 szStr    = NULL;
	BOOL		 bRetVal  = TRUE;

	_nNumFavorites++;

	menuInfo.cbSize = sizeof (menuInfo);
	menuInfo.fMask  = MIIM_STATE;
	if (GetMenuItemInfo (hMenu, 0, TRUE, &menuInfo))
	{
		if (menuInfo.fState == MF_GRAYED)
		{
			GOERROR(DeleteMenu (hMenu, 0, MF_BYPOSITION));
			uID -= 1;
			bSetInfo = TRUE;
		}
	}
	GOERROR(bRetVal = AppendMenu (hMenu, MF_STRING, uID, szText));

	if (bSetInfo)
	{
		menuInfo.cbSize = sizeof (MENUITEMINFO);
		menuInfo.fMask  = MIIM_DATA;
		GOERROR(szStr = new WCHAR [wcslen (szInfo) + 1]);
		wcscpy(szStr, szInfo);
		menuInfo.dwItemData = reinterpret_cast<DWORD>(szStr);
		GOERROR(SetMenuItemInfo (hMenu, uID, FALSE, &menuInfo));
	}
	return bRetVal;

ERR_RET:

	if (szStr)
		delete[] szStr;

	return FALSE;
}

HMENU CMainWnd::SetMenuInfo (HMENU hMenu, UINT uID, UINT uPos)
{
	HMENU		 hSubMenu  = NULL;
	LPTSTR		 szTmpStr1 = NULL;
	TCHAR	     szStr[MAX_REGKEY_LEN];
	TCHAR		 szTmpStr[17];
	MENUITEMINFO menuInfo;
		
	menuInfo.cbSize = sizeof (menuInfo);
	menuInfo.fMask  = MIIM_TYPE|MIIM_DATA;
	menuInfo.fType  = MFT_STRING;
	menuInfo.dwTypeData = new TCHAR[FAV_NAME_LEN];
	menuInfo.cch = FAV_NAME_LEN-1;
	uID -= 1;
	INIT_STR (szStr);

	if (!(hSubMenu = CreatePopupMenu()))
		return NULL;

	GetMenuItemInfo (hMenu, uID, FALSE, &menuInfo);
	if (menuInfo.dwItemData)
		wcscpy (szStr, reinterpret_cast<LPWSTR>(menuInfo.dwTypeData));

	GOERROR(DeleteMenu (hMenu, uID, MF_BYCOMMAND));

	if (0 < _nNumFavorites)
		_nNumFavorites--;

	GOERROR(LoadString (_Module.m_hInst,
				 		IDS_EMPTY_MENU_ITEM,
						szTmpStr,
						sizeof (szTmpStr)/sizeof (szTmpStr[0])));

	AddMenuItem (hSubMenu, uID, szTmpStr, NULL);
	EnableMenuItem(hSubMenu, uID, MF_BYCOMMAND | MF_GRAYED);
	AppendMenu (hMenu, MF_POPUP|MF_STRING,(UINT)hSubMenu,menuInfo.dwTypeData);

	if (szStr[0])
	{
		menuInfo.cbSize = sizeof (menuInfo);
		menuInfo.fMask  = MIIM_DATA;
		GOERROR(szTmpStr1 = new TCHAR [wcslen (szStr) + 1]);
		wcscpy (szTmpStr1, szStr);
		menuInfo.dwItemData = reinterpret_cast<DWORD>(szTmpStr1);
		GOERROR(SetMenuItemInfo (hMenu, uPos, TRUE, &menuInfo));
	}

	return hSubMenu;

ERR_RET:
	if (hSubMenu)
		DestroyMenu (hSubMenu);

	if (szTmpStr1)
		delete[] szTmpStr1;

	return NULL;
}
#endif // ndef NO_FAVORITES

#ifndef NOSTATUSBAR
BOOL CMainWnd::CreateStatusBar()
{
    if (!(_hWndStatus = CreateStatusWindow(WS_CHILD|WS_VISIBLE, _T("Ready"), _hWnd, ID_STATUSBAR)))
        return FALSE;


    RECT rc;
    GetClientRect(_hWndStatus, &rc);
    int naParts[5];
   
    naParts[4] = rc.right;
    naParts[3] = MAX(0, rc.right - 50);
    naParts[2] = MAX(0, rc.right - 50 - 20);
    naParts[1] = MAX(0, rc.right - 50 - 20 - 20);
    naParts[0] = MAX(0, rc.right - 50 - 20 - 20 - PROGRESSPANEWIDTH);

    SendMessage(_hWndStatus, SB_SETPARTS, (WPARAM )5, (LPARAM)naParts);

    SendMessage(_hWndStatus, SB_GETRECT, 1, (LPARAM )&rc);
    InsetRect(&rc, 1,1);
    _hWndProgress = CreateWindowEx(0, PROGRESS_CLASS, _T(""), WS_CHILD|WS_VISIBLE|PBS_SMOOTH,
                                   rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top,
                                   _hWndStatus, NULL, g_hInstance, NULL);
    if (!_hWndProgress)
        return FALSE;
    SendMessage(_hWndProgress, PBM_SETRANGE, 0, MAKELPARAM(0, 100));

    return TRUE;
}
#endif	//NOSTATUSBAR


void GetProxyOption()
{
	INTERNET_PER_CONN_OPTION_LIST	iOptionList;
	INTERNET_PER_CONN_OPTION		iOptions[3];
	ULONG							uSize = sizeof(iOptionList);

	iOptionList.dwSize			= uSize;
	iOptionList.pszConnection	= NULL;
	iOptionList.dwOptionCount	= 3;
	iOptionList.pOptions		= iOptions;

	// set proxy type direct or proxy server
	iOptions[0].dwOption	= INTERNET_PER_CONN_FLAGS;
	iOptions[1].dwOption	= INTERNET_PER_CONN_PROXY_SERVER;
	iOptions[2].dwOption	= INTERNET_PER_CONN_PROXY_BYPASS;
	
	if(InternetQueryOption(NULL,INTERNET_OPTION_PER_CONNECTION_OPTION ,(LPVOID)(&iOptionList),&uSize))
	{
		GlobalFree(iOptionList.pOptions[1].Value.pszValue);
		GlobalFree(iOptionList.pOptions[2].Value.pszValue);
	}
}

HWND CMainWnd::CreateBrowser()
{
    HRESULT hr;
    IUnknown *pUnk = NULL;
    IOleObject *pObject = NULL;


    if (!_pBrowser)
    {
		GetProxyOption();
        hr = CoCreateInstance(CLSID_WebBrowser, NULL,
                              CLSCTX_INPROC_SERVER | CLSCTX_INPROC_HANDLER,
                              IID_IUnknown, (LPVOID *)(&pUnk));
        if (hr)
            return FALSE;

        hr = pUnk->QueryInterface(IID_IOleObject, (LPVOID *)(&pObject));
        if (hr)
            goto Cleanup;

        DWORD dwFlags;
        hr = pObject->GetMiscStatus(DVASPECT_CONTENT, &dwFlags);
        if (hr)
            goto Cleanup;

        if (dwFlags & OLEMISC_SETCLIENTSITEFIRST)
        {
            IOleClientSite *pClientSite;
            hr = QueryInterface(IID_IOleClientSite, (LPVOID *)(&pClientSite));
            if (hr)
                goto Cleanup;

            hr = pObject->SetClientSite(pClientSite);
            pClientSite->Release();
            if (hr)
                goto Cleanup;
        }

        hr = Activate(pObject);
        if (hr)
            goto Cleanup;

        hr = _pObject->QueryInterface(IID_IWebBrowser2, (void **)&_pBrowser);
        if (hr)
            goto Cleanup;

        // See if there might be a url in lpszUrl
        hr = pUnk->QueryInterface(IID_IOleInPlaceActiveObject, (LPVOID *)(&_pIPActiveObj));
        if (S_OK!=hr)
			_pIPActiveObj = NULL;

        hr = S_FALSE;

        if (_fEmpty)
        {
            if (_lpszUrl)
            {
                LPTSTR lpszUrlAlpha = _lpszUrl;

                while (hr == S_FALSE && *lpszUrlAlpha != _T('\0'))
                {
                    if (_istalpha(*lpszUrlAlpha))
                        hr = S_OK;

                    lpszUrlAlpha++;
                }
            }

            if (hr == S_OK)
                _pBrowser->Navigate(_lpszUrl, NULL, NULL, NULL, NULL);
            else
                _pBrowser->GoHome();
        }
        hr = InitEvents();
    }

Cleanup:
    if (pUnk)
        pUnk->Release();
    if (pObject)
        pObject->Release();

    IOleWindow *pWnd = NULL;
    HWND hwndBrowser = NULL;

    if (_pBrowser)
    {
        hr = _pBrowser->QueryInterface(IID_IOleWindow, (LPVOID *)(&pWnd));
        if (hr)
            return NULL;
    }

    if (pWnd)
    {
        hr = pWnd->GetWindow(&hwndBrowser);
        pWnd->Release();
    }
    return hwndBrowser;
}

HRESULT CMainWnd::Activate(IOleObject *pObject)
{
    _pObject = pObject;
    _pObject->AddRef();

    RECT rc;
    ::GetClientRect(_hWnd, &rc);

    HRESULT hr;
    hr    =    _pObject->DoVerb(   OLEIVERB_UIACTIVATE,    NULL,    this,    0,    _hWnd,    &rc);

    if (hr)
        goto Cleanup;

    Cleanup:
    return hr;
}

HRESULT CMainWnd::InitEvents()
{
    HRESULT                     hr;
    IConnectionPointContainer  *pCPCont = NULL;
    DWebBrowserEvents          *pEvents = NULL;

    if (!_pBrowser)
        return S_FALSE;
    hr = _pBrowser->QueryInterface(IID_IConnectionPointContainer, (LPVOID *)&pCPCont);
    if (hr)
        return S_FALSE;
    hr = pCPCont->FindConnectionPoint(DIID_DWebBrowserEvents2, &_pCP);
    if (hr)
	{
		_pCP = NULL;
        goto Cleanup;
	}

    hr = QueryInterface(DIID_DWebBrowserEvents2, (LPVOID *)(&pEvents));
    if (hr)
        goto Cleanup;
    hr = _pCP->Advise(pEvents, &(_dwEventCookie));
    if (hr)
        goto Cleanup;


    Cleanup:
    if (pCPCont)
        pCPCont->Release();
    if (pEvents)
        pEvents->Release();
    return hr;
}

#ifndef NO_FAVORITES
VOID CMainWnd::AddToFavorites()
{
	// punt if theres no space left
	if (_nNumFavorites >= MAX_FAVORITES)
	{
		CFavListFullDlg dlg;
		dlg.Startup();
		return;
	} // if

	TCHAR	szTitle[FAV_NAME_LEN] = {0};
	_tcsncpy(szTitle, _szTitle, FAV_NAME_LEN-1);

	CAddFavoritesDlg dlg;
	if (!dlg.Startup(this,
					szTitle,
					GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0),FAVORITES_MENU_POSITION)))
		return;
	
	dlg.DoModal();
	if (dlg.IsRegistryChanged())
	{
		DeleteMenuItem (GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), FAVORITES_MENU_POSITION),
						FIRST_FAV_MENU_ITEM_POS);
	       _nNumFavorites = 0;
		_nLastFavoriteID = ID_FAVORITE_FIRST;
		PopulateFavoritesMenu();
	}
	
	// enable the organize favorites menu item if we have favorites
	if (_nNumFavorites > 0)
	{
	    HMENU hMenu = CommandBar_GetMenu(_hWndMenuBar, 0);
	    // EnableMenuItem(hMenu, ID_FAV_ORG, MF_BYCOMMAND | MF_ENABLED);
	} // if
} 

VOID CMainWnd::OrganizeFavorites()
{
	TCHAR	szTitle[FAV_NAME_LEN] = {0};
	COrgFavoritesDlg dlg (this);

	_tcsncpy(szTitle, _szTitle, FAV_NAME_LEN-1);
	dlg.DoModal();
	if (dlg.IsRegistryChanged())
	{
		DeleteMenuItem (GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), FAVORITES_MENU_POSITION),
						FIRST_FAV_MENU_ITEM_POS);
		_nNumFavorites = 0;
		_nLastFavoriteID = ID_FAVORITE_FIRST;
		PopulateFavoritesMenu();
	
	// disable the organize favorites menu item if we have no favorites
	    if (_nNumFavorites <= 0)
	    {
		HMENU hMenu = GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), FAVORITES_MENU_POSITION);
		
		AppendMenu (hMenu, MF_STRING, ID_FAV_PLACEHOLDER, _T("<Empty>"));
		EnableMenuItem(hMenu, ID_FAV_PLACEHOLDER, MF_BYCOMMAND | MF_GRAYED);
	    } // if
	}
} 

BOOL CMainWnd::GetUrlFromID(UINT nID, TCHAR* szUrl, UINT nMaxLen)
{
	MENUITEMINFO parentInfo;
	MENUITEMINFO menuInfo;
	TCHAR        szKey[MAX_REGKEY_LEN] = {0};
	TCHAR*		 pszValue				 = NULL;

	if ((nMaxLen <= 0) ||
		(!_pIFavStorage))
	{
		_hMenu = NULL;
		return FALSE;
	}

	if (_hMenu)
	{
		parentInfo.cbSize = sizeof(parentInfo);
		parentInfo.fMask = MIIM_DATA;
		GOERROR(GetMenuItemInfo(_hMenu, 0, TRUE, &parentInfo));
	}

	menuInfo.cbSize = sizeof(menuInfo);
	menuInfo.fMask = MIIM_TYPE;
	menuInfo.fType = MFT_STRING;
	GOERROR(menuInfo.dwTypeData = new TCHAR[nMaxLen]);
	menuInfo.cch = nMaxLen;
	if (_hMenu)
	{
		GOERROR(GetMenuItemInfo(_hMenu, nID, FALSE, &menuInfo));
		wsprintf (szKey, L"%s\\%s",reinterpret_cast<LPWSTR>(parentInfo.dwItemData),menuInfo.dwTypeData);
	}
	else
	{
		GOERROR(GetMenuItemInfo(GetSubMenu(CommandBar_GetMenu(_hWndMenuBar, 0), FAVORITES_MENU_POSITION),
								nID,
								FALSE,
								&menuInfo));
		wsprintf (szKey, L"%s\\%s",REG_KEY_BROWSER_FAVORITES,menuInfo.dwTypeData);
	}

	if (menuInfo.dwTypeData)
		free(menuInfo.dwTypeData);

	_hMenu = NULL;
	GOERROR(pszValue = _pIFavStorage->GetURL(szKey));
	wcscpy(szUrl,pszValue);
	delete[] pszValue;
	return TRUE;

ERR_RET:

	if (menuInfo.dwTypeData)
		free(menuInfo.dwTypeData);

	_hMenu = NULL;
	return FALSE;

} // GetUrlFromID

LPTSTR CMainWnd::GetRegistryKeyString(HKEY hKey, LPCTSTR lptValueName)
{
    DWORD	size;
	DWORD	type;
    PBYTE	pByte;

	pByte = NULL;
	if (RegQueryValueEx(hKey, lptValueName, NULL, &type, NULL, &size) == 
		ERROR_SUCCESS && (type == REG_SZ || type == REG_MULTI_SZ))
	{
		pByte = (PBYTE) malloc(size);
		if ((!pByte) || (RegQueryValueEx(hKey, lptValueName, NULL, NULL, pByte, &size) != ERROR_SUCCESS) || (pByte[0] == 0))
		{
			free(pByte);
			pByte = NULL;
		}
	}
	return (LPTSTR) pByte;
}
#endif // ndef NO_FAVORITES

HRESULT HandleNewWindow2(LPTSTR lpszUrl, DISPPARAMS FAR* pdparams)
{

        HANDLE hThread;
        CMainWnd *pNewWnd;
        IDispatch *pDispatch;
        HRESULT hr = S_OK;

        pNewWnd = new CMainWnd;

        if (!pNewWnd)
        {
            return E_OUTOFMEMORY;
        }

        pNewWnd->_lpszUrl = lpszUrl;
        if(!pdparams)
            pNewWnd->_fEmpty = TRUE;
            
        pNewWnd->hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);

        if (!pNewWnd->hEvent)
        {

        }

        InterlockedIncrement(&glThreadCount);

        hThread = CreateThread(NULL, g_dwMainWindowStackSize, NewWindow, (LPVOID)pNewWnd, STACK_SIZE_PARAM_IS_A_RESERVATION, NULL);

        if (!hThread)
        {
            delete pNewWnd;
            InterlockedDecrement(&glThreadCount);
            return E_OUTOFMEMORY;
        }

        WaitForSingleObject(pNewWnd->hEvent, INFINITE);

        if(pdparams)
        {
            if (pNewWnd->_pBrowser)
            {
                hr = pNewWnd->_pBrowser->QueryInterface(IID_IDispatch, (LPVOID *)(&pDispatch));
            }
            else
            {
                hr = E_FAIL;
                pDispatch = NULL;
            }

            *(pdparams->rgvarg[0].pboolVal) = 0;
            *(pdparams->rgvarg[1].ppdispVal) = pDispatch;
        }
        return hr;
}

STDMETHODIMP CMainWnd::Invoke (
                              DISPID dispidMember,
                              REFIID riid,
                              LCID lcid,
                              WORD wFlags,
                              DISPPARAMS FAR* pdparams,
                              VARIANT FAR* pvarResult,
                              EXCEPINFO FAR* pexcepinfo,
                              UINT FAR* puArgErr
                              )
{
    switch (dispidMember)
    {
#ifndef NOSTATUSBAR
    case DISPID_STATUSTEXTCHANGE:
        {
            if (pdparams && pdparams->rgvarg[0].vt == VT_BSTR)
            {
                // NULL terminate the string
                TCHAR   szStatusText[128];

                wcsncpy(szStatusText, pdparams->rgvarg[0].bstrVal, 128);
                szStatusText[128 - 1] = 0;
                SendMessage(_hWndStatus, SB_SETTEXT, 0, (LPARAM)szStatusText);
            }
            break;
        }

    case DISPID_SETSECURELOCKICON:
        if(pdparams && pdparams->rgvarg[0].vt == VT_I4)
        {
                TCHAR szFakeIcon[4];
                if(pdparams->rgvarg[0].lVal != 0)
                {
                    wcscpy(szFakeIcon, L"S");
                }
                else
                {
                    wcscpy(szFakeIcon, L" ");
                }
                SendMessage(_hWndStatus, SB_SETTEXT, 2, (LPARAM)szFakeIcon);
        }
        break;

    case DISPID_PROGRESSCHANGE:
        {
            LONG lProgMax = pdparams->rgvarg[0].lVal;
            LONG lProg = pdparams->rgvarg[1].lVal;
            UINT nPercent = (lProg == -1) ? 0 : (lProgMax) ? lProg*100/lProgMax : 0;
            SendMessage(_hWndProgress, PBM_SETPOS, nPercent, 0);
        }
        break;
#endif	//NOSTATUSBAR

#ifndef NOCOMMANDBAR
    case DISPID_NAVIGATECOMPLETE2:
        if (_bTyped)
        {
            UpdateUrlList(pdparams->rgvarg[0].pvarVal->bstrVal);
            _bTyped = FALSE;
        }
        else
        {
            // add the URL which was navigated to by clicking to the URLList combo-box.
            BSTR bstrUrl = NULL;
            _pBrowser->get_LocationURL(&bstrUrl);
            UpdateUrlList((LPCTSTR)bstrUrl);
            SysFreeString(bstrUrl);
        }
        break;

    case DISPID_COMMANDSTATECHANGE:
    {
        DWORD dwState;
        DWORD dwId = -1;
        
        switch ((LONG)pdparams->rgvarg[1].lVal) {
        case CSC_NAVIGATEFORWARD :
            dwId = ID_GO_FORWARD;
            break;
        case CSC_NAVIGATEBACK :
            dwId = ID_GO_BACK;
            break;
        }

        if (dwId != -1) {
            dwState = SendMessage (_hWndToolBar, TB_GETSTATE, dwId, 0);
            if ((BOOL)pdparams->rgvarg[0].boolVal) {
                dwState |= TBSTATE_ENABLED;
            } else {
                dwState &= ~(TBSTATE_ENABLED);
            }
            SendMessage (_hWndToolBar, TB_SETSTATE, dwId, dwState);
        }
        break;
    }
#endif	//NOCOMMANDBAR

    case DISPID_TITLECHANGE:
        {
            if (pdparams && pdparams->rgvarg[0].vt == VT_BSTR)
            {
                TCHAR szTitle[MAXLEN*2];
                _tcsncpy(_szTitle, pdparams->rgvarg[0].bstrVal, MAXLEN);
                _tcsncpy(szTitle, pdparams->rgvarg[0].bstrVal, MAXLEN);
                _tcscat(szTitle, LOADSTRING(IDS_HYPHEN));
                _tcscat(szTitle, LOADSTRING(IDS_IE));

                SetWindowText(_hWnd, szTitle);
            }
            break;
        }

    case DISPID_ONQUIT:
        PostMessage(_hWnd, WM_CLOSE, 0, 0L);
        break;

#ifndef NOCOMMANDBAR
    case DISPID_DOWNLOADBEGIN:
        _pthrdAnimation->StartAnimation();
        break;
    case DISPID_DOWNLOADCOMPLETE:
    case DISPID_DOCUMENTCOMPLETE:
        _pthrdAnimation->StopAnimation();
        SendMessage(_hWndProgress, PBM_SETPOS, 0, 0);
		
		// Make sure that the current fontsize is checked.
		ChangeFontSize(0);
        break;
#endif	//NOCOMMANDBAR

    case DISPID_NEWWINDOW2:
        return HandleNewWindow2(NULL, pdparams);
       
        break;


    case DISPID_PROPERTYCHANGE:
    case DISPID_BEFORENAVIGATE2:
        break;
    default:
        break;
    }
    return S_OK;
}


//=============================================================================
//CMainWnd::HandleCommand(WPARAM wParam, LPARAM lParam)
//=============================================================================
#ifndef NOCOMMANDBAR
EXTERN_C const GUID CGID_MSHTML;
LRESULT CMainWnd::HandleCommand(WPARAM wParam, LPARAM lParam)
{

    UINT nID = LOWORD(wParam);
    switch (nID)
    {
    case ID_ADDRESSBAR:
        if (HIWORD(wParam) == CBN_SELENDOK)
        {
            SelChange();
            return 0;
        }
        break;
#ifndef NOFIND
    case ID_FILE_FIND:
        {
            FindString();
            return 0;
        }
#endif	//NOFIND

#ifndef NOPRINT
    case ID_FILE_PRINT:
        {
			_pBrowser->ExecWB(OLECMDID_PRINT, OLECMDEXECOPT_DODEFAULT, NULL, NULL);	
            return 0;
        }
#endif	//NOPRINT

#ifndef NOOFFLINE
	case ID_FILE_OFFLINE:
		{
			INTERNET_CONNECTED_INFO ci;
            DWORD dwBufferLength;

			memset(&ci, 0, sizeof(ci));
            dwBufferLength = sizeof(ci);
			InternetQueryOption(NULL, INTERNET_OPTION_CONNECTED_STATE, &ci, &dwBufferLength);
            ci.dwConnectedState = (ci.dwConnectedState & INTERNET_STATE_DISCONNECTED_BY_USER) ? INTERNET_STATE_CONNECTED : INTERNET_STATE_DISCONNECTED_BY_USER;
			InternetSetOption(NULL, INTERNET_OPTION_CONNECTED_STATE, &ci, sizeof(ci));
			return 0;
		}
#endif //NOOFLINE

    case ID_FILE_OPEN:
          {
			_pBrowser->ExecWB(OLECMDID_OPEN, OLECMDEXECOPT_DODEFAULT, NULL, NULL);	
                    return 0;
          }
          break;

    case ID_FILE_SAVEAS:
          {
                     VARIANT vTitle;
                     BSTR bstrTitle = SysAllocString(_szTitle);
                    if(bstrTitle)
                    {
                        WCHAR *pch = bstrTitle;
                        // Strip dots. Perhaps find a more locale friendly way of doing this?
                        while(*pch)
                        {
                                if(*pch == L'.')  
                                       *pch = L'_';
                                pch++;
                         }
                         vTitle.vt = VT_BSTR;
                         vTitle.bstrVal = bstrTitle;
    			    _pBrowser->ExecWB(OLECMDID_SAVEAS, OLECMDEXECOPT_DODEFAULT, &vTitle, NULL);	
    			    SysFreeString(bstrTitle);
    		      }
                    return 0;
          }
//HJG
#ifndef NO_CLOSE
	case ID_FILE_CLOSE:
		_pBrowser->ExecWB(OLECMDID_CLOSE, OLECMDEXECOPT_DODEFAULT, NULL, NULL);
		// PostMessage(_hWndBrowser, WM_CLOSE, 0, 0L);
		return 0;
#endif
    case ID_EDIT_CUT:
		if (GetFocus()== _hWndAddressEdit)
			SendMessage(_hWndAddressEdit, WM_CUT, WPARAM(0), LPARAM(0));
		else
			_pBrowser->ExecWB(OLECMDID_CUT, OLECMDEXECOPT_DONTPROMPTUSER,
                          NULL, NULL);
        return 0;

    case ID_EDIT_COPY:
		if (GetFocus()== _hWndAddressEdit)
			SendMessage(_hWndAddressEdit, WM_COPY, WPARAM(0), LPARAM(0));
		else
    	    _pBrowser->ExecWB(OLECMDID_COPY, OLECMDEXECOPT_DONTPROMPTUSER,
                          NULL, NULL);
        return 0;
    case ID_EDIT_PASTE:
		if (GetFocus() == _hWndAddressEdit)
			SendMessage(_hWndAddressEdit, WM_PASTE, WPARAM(0), LPARAM(0));
		else
	        _pBrowser->ExecWB(OLECMDID_PASTE, OLECMDEXECOPT_DONTPROMPTUSER,
                          NULL, NULL);
        return 0;



    case ID_VIEW_FONTSLARGEST:
    case ID_VIEW_FONTSLARGER:
    case ID_VIEW_FONTSMEDIUM:
    case ID_VIEW_FONTSSMALLER:
    case ID_VIEW_FONTSSMALLEST:
        ChangeFontSize(LOWORD(wParam));
        return 0;

#ifndef NO_FAVORITES
	case ID_FAV_ADD:
		AddToFavorites();
		return 0;

	case ID_FAV_ORG:
		OrganizeFavorites();
		return 0;
#endif

#ifndef NOOPTIONS
    case ID_VIEW_OPTIONS:
    {
       SHELLEXECUTEINFO sei;
       sei.cbSize = sizeof(sei);
       sei.fMask = SEE_MASK_FLAG_NO_UI;
       sei.hwnd = _hWnd;
       sei.lpVerb = NULL;
       sei.lpFile = L"ctlpnl";
       sei.lpParameters = L"inetcpl.cpl";
       sei.lpDirectory = NULL;
       sei.nShow = SW_SHOWNORMAL;

       ShellExecuteEx(&sei);

    }
       //DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_OPTIONS), _hWnd, (DLGPROC)OptionsDlgProc);
        return 0;
#endif	//NOOPTIONS

	// a button was pressed
	//HJG
	case ID_GO_EXIT:
				   //Passwortabfrage
/*					a = DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_PASSWORD_DLG), _hWnd,  (DLGPROC)PasswordDlgProc);
					if (a == IDCANCEL)
					{
						MessageBeep(MB_ICONEXCLAMATION);
						return 0;
					}
*/		_pBrowser->ExecWB(OLECMDID_CLOSE, OLECMDEXECOPT_DODEFAULT, NULL, NULL);
		// PostMessage(_hWndBrowser, WM_CLOSE, 0, 0L);
		return 0;
				   break;

	//HJG

    case ID_GO_BACK:
        _pBrowser->GoBack();
        return 0;
    case ID_GO_FORWARD:
        _pBrowser->GoForward();
        return 0;
    case ID_GO_HOME:
        _pBrowser->GoHome();
        return 0;
    case ID_GO_SEARCH:
        _pBrowser->GoSearch();
        return 0;
    case ID_VIEW_REFRESH:
        _pBrowser->Refresh();
        return 0;
    case ID_VIEW_STOP:
        _pBrowser->Stop();
        return 0;
    case ID_FOCUS_URL:
        if (_hWndAddressEdit)
        {
            SetFocus(_hWndAddressEdit);
        }
        return 0;
    default:

#ifndef NO_FAVORITES
    		// handle favorite menu items
	if (nID >= ID_FAVORITE_FIRST && nID <= ID_FAVORITE_LAST)
	{
			TCHAR szUrl[MAX_URL] = {0};

			if (GetUrlFromID(nID, szUrl, MAX_URL))
				_pBrowser->Navigate(szUrl, NULL, NULL, NULL, NULL); 

	  } 
#endif

        // if this is from the encoding popup
        if (IDM_MIMECSET__FIRST__ <= nID && IDM_MIMECSET__LAST__ >= nID)
        {
		    if (_pBrowser)
            {
                IOleCommandTarget* pcmdt;
	            if (SUCCEEDED(_pBrowser->QueryInterface(IID_IOleCommandTarget, (LPVOID*) &pcmdt)))
	            {
                    pcmdt->Exec(&CGID_MSHTML, nID, 0, NULL, NULL);
	                pcmdt->Release();
                }
            }
        }
        return 0;
    }
    return TRUE;
}

VOID CMainWnd::ChangeFontSize(WORD wSize)
{
    VARIANT vaSize;
    vaSize.vt = VT_I4;
	if (0 == wSize) {
		// Setting initial value
		vaSize.vt = 0;
		HRESULT hr = _pBrowser->ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
									   NULL, &vaSize);
		wSize = ID_VIEW_FONTSSMALLEST + (WORD)vaSize.lVal;
	} else {
		vaSize.lVal = wSize - ID_VIEW_FONTSSMALLEST;
		HRESULT hr = _pBrowser->ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
									   &vaSize, NULL);
	}
    HMENU hMenu = CommandBar_GetMenu(_hWndMenuBar, 0);
    BOOL bRet = CheckMenuRadioItem(hMenu, ID_VIEW_FONTSSMALLEST ,ID_VIEW_FONTSLARGEST, wSize, MF_BYCOMMAND);
    CommandBar_DrawMenuBar(_hWndMenuBar, 0);
}
#endif	//NOCOMMANDBAR




#ifndef NOCOMMANDBAR
LRESULT CALLBACK CMainWnd::AddressEditProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{


    CMainWnd *pMainWnd = (CMainWnd*)GetWindowLong(hwnd, GWL_USERDATA);

    if (!pMainWnd)
    {
        return 0;
    }

    if ((message == WM_CHAR) && (wParam == VK_RETURN))
    {
        TCHAR szString[MAXLEN];
	int iSpaces = 0;
        GetWindowText(hwnd, szString, MAXLEN);
        // Remove preceding spaces.
        for (iSpaces=0; szString[iSpaces] == _T(' '); iSpaces++);
        pMainWnd->_pBrowser->Navigate(&szString[iSpaces], NULL, NULL, NULL, NULL);
        pMainWnd->_bTyped = TRUE;
        return 0;
    }
    else
        return CallWindowProc(pMainWnd->_lpfnAddressEditProc, hwnd, message, wParam, lParam);
}
#endif	//NOCOMMANDBAR

EXTERN_C const GUID CGID_ShellDocView;

LRESULT CALLBACK CMainWnd::MainWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    CMainWnd *pMainWnd = (CMainWnd*)GetWindowLong(hwnd, GWL_USERDATA);
	HWND hwndTaskBar;

    if (pMainWnd)
    {
        switch (message)
        {

        case WM_CLOSE:
            pMainWnd->Close();
            break;
        case WM_DESTROY:
#ifdef USEFULLSCREEN
			hwndTaskBar = FindWindow(L"HHTASKBAR", NULL);
			if (hwndTaskBar!=NULL)
				EnableWindow(hwndTaskBar, true);
#endif
            ::PostQuitMessage(0);
            break;
	case WM_SETFOCUS:
		if (pMainWnd->_pIPActiveObj)
		{
			pMainWnd->_pIPActiveObj->OnFrameWindowActivate(LOWORD(wParam) != WA_INACTIVE);
		}
		return 0;

        case WM_COMMAND:
#ifndef NOCOMMANDBAR
            return pMainWnd->HandleCommand(wParam, lParam);
#else
			if ( LOWORD(wParam)==ID_FILE_CLOSE ) //process ALT+Y to close, UINT nID = LOWORD(wParam);
				pMainWnd->Close();//::PostQuitMessage(0);

#endif  //NOCOMMANDBAR
            break;

        case WM_NOTIFY:
            if(wParam== ID_COMMANDBAND) {		
				NMREBAR * pnm = (NMREBAR*)lParam;
				switch(pnm->hdr.code) {
					case RBN_LAYOUTCHANGED:
						break;

					case RBN_HEIGHTCHANGE:
					{
					      RECT rcMain, rcBrowse, rcStatus;
					      
					      int iHeight = SendMessage(pnm->hdr.hwndFrom, RB_GETBARHEIGHT, 0, 0);

                                         if(pMainWnd->_hWndStatus)
                                         {
                                                GetClientRect(pMainWnd->_hWndStatus, &rcStatus);
                                         }
                                         else
                                         {
                                                rcStatus.top = rcStatus.bottom = 0;
                                         }
                                         GetClientRect(pMainWnd->_hWnd, &rcMain);
                                         GetClientRect(pMainWnd->_hWndBrowser, &rcBrowse);
                                         int iBrowseHeight = (rcMain.bottom - rcMain.top) - iHeight -(rcStatus.bottom - rcStatus.top);

                                         if(iBrowseHeight != (rcBrowse.bottom - rcBrowse.top))
                                         {
					            SetWindowPos(pMainWnd->_hWndBrowser,  
					                            NULL,
					                            0,
					                            iHeight,
					                            rcMain.right-rcMain.left, 
					                            iBrowseHeight, 
					                            SWP_NOZORDER);					
                                        }
					}
					break;
			}
	    }
	    break;

        case WM_INITMENUPOPUP:

            {
                // for WM_INITMENUPOPUP
                OLECMD rgcmd[] = {
                   { OLECMDID_CUT, 0 },
                   { OLECMDID_COPY, 0 },
                   { OLECMDID_PASTE, 0 }
                };

                // check/uncheck the offline state
                INTERNET_CONNECTED_INFO ci;
                DWORD dwBufferLength;

                memset(&ci, 0, sizeof(ci));
                dwBufferLength = sizeof(ci);
                InternetQueryOption(NULL, INTERNET_OPTION_CONNECTED_STATE, &ci, &dwBufferLength);
                CheckMenuItem((HMENU)wParam, ID_FILE_OFFLINE, MF_BYCOMMAND | (ci.dwConnectedState & INTERNET_STATE_DISCONNECTED_BY_USER) ? MF_CHECKED : MF_UNCHECKED);

                // enable / disable edit menu commands
                IOleCommandTarget* pcmdt;
                int i;

				if (GetFocus()==ghWndAddressEdit)
				{
					DWORD dwStart, dwEnd;
					
					SendMessage(ghWndAddressEdit, EM_GETSEL, (WPARAM)(&dwStart), (LPARAM)(&dwEnd));
					if (dwEnd - dwStart > 1)
					{
						EnableMenuItem((HMENU)wParam, ID_EDIT_CUT, MF_BYCOMMAND | MF_ENABLED);
						EnableMenuItem((HMENU)wParam, ID_EDIT_COPY, MF_BYCOMMAND | MF_ENABLED);
					}
					else
					{
						EnableMenuItem((HMENU)wParam, ID_EDIT_CUT, MF_BYCOMMAND | MF_GRAYED);
						EnableMenuItem((HMENU)wParam, ID_EDIT_COPY, MF_BYCOMMAND | MF_GRAYED);
					}					
					if (IsClipboardFormatAvailable(CF_UNICODETEXT) || IsClipboardFormatAvailable(CF_TEXT))
					{
						EnableMenuItem((HMENU)wParam, ID_EDIT_PASTE, MF_BYCOMMAND | MF_ENABLED);
					}					
					else
					{
						EnableMenuItem((HMENU)wParam, ID_EDIT_PASTE, MF_BYCOMMAND | MF_GRAYED);
					}					
				}
				else
				{
				   if (!pMainWnd->_pBrowser)
    	               break;

	               	if (SUCCEEDED(pMainWnd->_pBrowser->QueryInterface(IID_IOleCommandTarget, (LPVOID*) & pcmdt)))
	              	{
	                    pcmdt->QueryStatus(NULL, _countof(rgcmd), rgcmd, NULL);
	                    pcmdt->Release();
	   	            }
	                for (i=0; i<_countof(rgcmd); i++)
	                {
	                    EnableMenuItem((HMENU)wParam, ID_EDIT_CUT + i, rgcmd[i].cmdf & OLECMDF_ENABLED ?
	                        (MF_BYCOMMAND | MF_ENABLED) : ( MF_BYCOMMAND| MF_GRAYED));
	                }
				}
#ifndef NO_FAVORITES
                MENUITEMINFO mInfo;
                mInfo.cbSize = sizeof(mInfo);
                mInfo.fMask = MIIM_DATA;
                GetMenuItemInfo((HMENU)wParam, 0, TRUE, &mInfo);
		 pMainWnd->_hMenu = NULL;
		 if (mInfo.dwItemData != NULL)
		 {
			// Store this for tracking purpouse
			if (!_tcsncmp (reinterpret_cast<LPTSTR>(mInfo.dwItemData), 
						   REG_KEY_BROWSER_FAVORITES,
						   _tcslen (REG_KEY_BROWSER_FAVORITES)))
				pMainWnd->_hMenu = (HMENU)wParam;
		 }
#endif				
                // provide encoding menu
			    if (!pMainWnd->_pBrowser)
    	            break;

                // if this is not the view menu, bail out early
                MENUITEMINFO mii;
                

                mii.cbSize = sizeof(mii);
                mii.fMask = MIIM_ID;

                GetMenuItemInfo((HMENU)wParam, 3, TRUE, &mii);
                if (mii.wID != ID_VIEW_STOP)
                    break;

	            if (SUCCEEDED(pMainWnd->_pBrowser->QueryInterface(IID_IOleCommandTarget, (LPVOID*) & pcmdt)))
	            {
                    VARIANTARG v = {0};
                    HRESULT hr;

                    hr = pcmdt->Exec(&CGID_ShellDocView, 27/*SHDVID_GETMIMECSETMENU*/, 0, NULL, &v);
                    if (S_OK == hr)
                    {
                        
                        UINT uItem = 0;
                        HMENU hmenuFonts = (HMENU)v.lVal;
                        WCHAR szTitle[50];
                        
                        mii.fMask = MIIM_TYPE;
                        mii.fType = MFT_STRING;
                        mii.dwTypeData = szTitle;
                        mii.cch = 50;

                        GetMenuItemInfo((HMENU)wParam, 1, TRUE, &mii);

                        HMENU hmenuOldFonts = GetSubMenu((HMENU)wParam, 1);
                        RemoveMenu((HMENU)wParam, 1, MF_BYPOSITION);
                        DestroyMenu(hmenuOldFonts);
                        InsertMenu((HMENU)wParam, 1, MF_POPUP|MF_BYPOSITION, (UINT)hmenuFonts, szTitle);
                    }

	                pcmdt->Release();
                }

			}
            break;

        default:
            break;
        }
    }
    return DefWindowProc(hwnd, message, wParam, lParam);
}

BOOL CMainWnd::PreTranslateMessage(LPMSG pMsg)
{
    if (TranslateAccelerator(_hWnd, _hAccelTbl, pMsg))  //first shot to the main frame
        return TRUE;

#ifndef NOCOMMANDBAR
    if (pMsg->hwnd == _hWndAddressEdit)
        return FALSE;
#endif	//NOCOMMANDBAR

    //then to mshtml
    if (_pIPActiveObj && pMsg->message >= WM_KEYFIRST && pMsg->message <= WM_KEYLAST)
    {
        HRESULT hr = _pIPActiveObj->TranslateAccelerator(pMsg);
        return (hr != S_FALSE);
    }
    return FALSE;
}

#ifndef NOCOMMANDBAR
VOID CMainWnd::LoadTypedUrls()
{
    DWORD dwType = 0;
    HKEY hKey = NULL;

    LONG lRes = RegOpenKeyEx(HKEY_CURRENT_USER, szTypedUrls, 0, KEY_QUERY_VALUE, &hKey);
    if (hKey && lRes==ERROR_SUCCESS)
    {
        DWORD dwValNames, dwMaxValNameLen, dwMaxValDataLen;
        lRes = RegQueryInfoKey(hKey, NULL, NULL, NULL, NULL, NULL, NULL,
                               &dwValNames, &dwMaxValNameLen, &dwMaxValDataLen, NULL, NULL);
        if (lRes==ERROR_SUCCESS)
        {
            TCHAR pszValName[10];
            TCHAR pszValData[MAXLEN];
            DWORD cbValName, cbData;
            for (DWORD i=dwValNames; i>0; i--)
            {
                cbData = sizeof(TCHAR)*MAXLEN;
                lRes = RegEnumValue(hKey, i-1, pszValName, &cbValName, 0,
                                    &dwType, (BYTE *)pszValData, &cbData);
				if (lRes ==ERROR_SUCCESS)
					SendMessage(_hWndAddressCombo, CB_INSERTSTRING, 0, (LPARAM )pszValData);
            }
        }
    }

}

VOID CMainWnd::UpdateUrlList(LPCTSTR lpszUrl)
{

    int i = SendMessage(_hWndAddressCombo, CB_FINDSTRINGEXACT,
                        (WPARAM )-1, (LPARAM )lpszUrl);
    if (i == CB_ERR)
    {
        SendMessage(_hWndAddressCombo, CB_INSERTSTRING, 0, (LPARAM )lpszUrl);
        SendMessage(_hWndAddressCombo, CB_SETCURSEL,
                    (WPARAM)0, (LPARAM)0);
    }
    else
    {
        SendMessage(_hWndAddressCombo, CB_SETCURSEL,
                    (WPARAM)i, (LPARAM)0);
    }

    if ( (SendMessage(_hWndAddressCombo, CB_GETCOUNT, 0, 0L)) > MAXURLS)
    {
        SendMessage(_hWndAddressCombo, CB_DELETESTRING, MAXURLS-1, 0L);
    }
}

VOID CMainWnd::SelChange()
{
    TCHAR *szUrl;
    int nRet, nRet1 = SendMessage(_hWndAddressCombo, CB_GETCURSEL, 0, 0L);
    if (nRet1==CB_ERR)
        return ;
	nRet = SendMessage(_hWndAddressCombo, CB_GETLBTEXTLEN, nRet1, 0L); 
	if (nRet==CB_ERR)
		return;
	szUrl = new TCHAR[nRet+1];
	if (szUrl)  {
		SendMessage(_hWndAddressCombo, CB_GETLBTEXT, nRet1, (LPARAM )szUrl);
		_pBrowser->Navigate(szUrl, NULL, NULL, NULL, NULL);
		delete szUrl;
	}
}
#endif	//NOCOMMANDBAR


VOID CMainWnd::Close()
{
    RETAILMSG(1,(L"IESAMPLE CMainWnd::Close"));

    // Tell the shell to die off
    SendMessage(_hWndBrowser, WM_CLOSE, 0,0);
#ifndef NOCOMMANDBAR
    HKEY hKey = NULL;
    TCHAR *szString=NULL;	int cbString = 0;
    DWORD dwDisp;

    LONG lRes = RegCreateKeyEx(HKEY_CURRENT_USER, szTypedUrls, 0, NULL, 0, 0, 0, &hKey, &dwDisp);
    if (hKey && lRes==ERROR_SUCCESS)
    {
        int nCount = SendMessage(_hWndAddressCombo, CB_GETCOUNT, 0, 0L);
        int nLen;
        TCHAR szValueName[7];
        for (int i=0; i<nCount; i++)
        {
			nLen = SendMessage(_hWndAddressCombo, CB_GETLBTEXTLEN, i, 0L); 
			if (nLen==CB_ERR)  {
				delete szString;  cbString = 0;
				return;
			}
			if (nLen > cbString)  {
				delete szString;  cbString = 0;
				szString = new TCHAR[nLen+1];
				if (!szString)
					break;
				cbString = nLen;
			}
            nLen = SendMessage(_hWndAddressCombo, CB_GETLBTEXT, i, (LPARAM )szString);
            wsprintf(szValueName, _T("url%d"), i+1);
            lRes = RegSetValueEx(hKey, szValueName, 0, REG_SZ, (BYTE *)szString, (_tcslen(szString)+1)*sizeof(TCHAR));
        }
    }
	delete szString; cbString = 0;

    SetWindowLong(_hWndAddressEdit, GWL_WNDPROC, (LONG)_lpfnAddressEditProc);
#endif	//NOCOMMANDBAR

	if(_pIPActiveObj)
	{
		_pIPActiveObj->Release();
	}
    _pIPActiveObj = NULL;
	if(_pCP)
	{
		_pCP->Unadvise(_dwEventCookie);
		_pCP->Release();
	}
    _pCP = NULL;
	if(_pObject)
	{
		_pObject->Close(FALSE);
		_pObject->Release();
	}
    _pObject = NULL;
	if(_pBrowser)
	    _pBrowser->Release();
    _pBrowser = NULL;

#ifndef NOCOMMANDBAR
     if(_himlCmdBand)
     {
                ImageList_Destroy(_himlCmdBand);
                _himlCmdBand = NULL;
      }
#endif
}

#ifdef OEMCONTROLS
LRESULT CALLBACK OemComboProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
    case WM_PAINT:
        {
            PAINTSTRUCT ps;
            BeginPaint(hwnd, &ps);
            HDC hDC = GetDC(hwnd);
            SelectObject(hDC, GetStockObject(BLACK_PEN));
            SelectObject(hDC, GetStockObject(SYSTEM_FONT));
            RECT rc;
            GetClientRect(hwnd, &rc);
            TCHAR szText[] = _T("OEM Combo Box");
            DrawText(hDC, szText, _tcslen(szText),&rc, DT_LEFT|DT_TOP);
            EndPaint(hwnd, &ps);
            return 0;
        }
    default:
        break;
    }
    return DefWindowProc(hwnd, message, wParam, lParam);

}

LRESULT CALLBACK OemListBoxProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
    case WM_PAINT:
        {
            PAINTSTRUCT ps;
            BeginPaint(hwnd, &ps);
            HDC hDC = GetDC(hwnd);
            SelectObject(hDC, GetStockObject(BLACK_PEN));
            SelectObject(hDC, GetStockObject(SYSTEM_FONT));
            RECT rc;
            GetClientRect(hwnd, &rc);
            TCHAR szText[] = _T("OEM Combo Box");
            DrawText(hDC, szText, _tcslen(szText),&rc, DT_LEFT|DT_TOP);
            EndPaint(hwnd, &ps);
            return 0;
        }
    default:
        break;
    }
    return DefWindowProc(hwnd, message, wParam, lParam);

}


BOOL RegisterOemControls()
{
    WNDCLASS        wc;
    wc.style         = 0;
    wc.lpfnWndProc   = (WNDPROC)OemComboProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = g_hInstance;
    wc.hIcon         = NULL;
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetSysColorBrush(COLOR_WINDOW);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = TEXT("combobox");

    if (!(RegisterClass(&wc)))
        return FALSE;

    wc.lpfnWndProc = (WNDPROC )OemListBoxProc;
    wc.lpszClassName = TEXT("listbox");

    if (!(RegisterClass(&wc)))
        return FALSE;

    return TRUE;

}

#endif	//OEMCONTROLS

#ifndef NOFIND
HRESULT CMainWnd::FindString()
{
    LPDISPATCH pDisp = NULL;
    LPOLECOMMANDTARGET pCmdTarg = NULL;
    HRESULT sts = S_OK;
    VARIANTARG var;

#define GOERROR_S(bNotCond)  {if (!(bNotCond)) goto errorS; }

    if (!_pBrowser)
        return S_OK;
    sts = _pBrowser->get_Document(&pDisp);
    GOERROR_S(pDisp);
    sts = pDisp->QueryInterface(IID_IOleCommandTarget, (LPVOID*)&pCmdTarg);
    GOERROR_S(pCmdTarg);

    var.vt   = VT_I4;
    var.lVal = 0;
    sts = pCmdTarg->Exec(
                        NULL,
                        OLECMDID_FIND,
                        MSOCMDEXECOPT_PROMPTUSER,
                        NULL,
                        &var);

    errorS:
    if (pCmdTarg)
        pCmdTarg->Release(); // release document's command target
    if (pDisp)
        pDisp->Release();    // release document's dispatch interface
    return sts;
}
#endif	//NOFIND

